%% Nyquist Plot Example
% This file generates a Nyquist plot for a first order system using the
% Matlab command "nyquist".

%% Transfer Function
b0 = 2;
a0 = 4;
G = tf(b0,[1 a0]);

%% Nyquist Plot

% Generate plot directly with Nyquist command
figure(1)
clf;
nyquist(G);

% Use Nyquist command to compute real and imaginary data. Use this
% data to create own Nyquist plot.
% Note: The "nyquist" command returns the real and imaginary parts in a 
% three-dimensional array with dimensions Ny-by-Nu-by-Nw where Ny/Nu/Nw
% are the number of outputs, inputs, and frequencies.  For single-input
% single-output systems this is 1-by-1-by-Nw.  The ":" operation, e.g.
% Gre(:), converts this data into an Nw-by-1 column vector.
[Gre,Gim,ww] = nyquist(G);
Gre = Gre(:);
Gim = Gim(:);

figure(2)
clf;

plot(Gre,Gim,'b',Gre,-Gim,'r-.')
xlabel('Real Part');
ylabel('Imag. Part');
title('Nyquist Diagram');
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

%% Plot arrowheads at certain frequencies

w = [1 16];
for i=1:numel(w)
    [Gre,Gim] = nyquist(G,[w(i) 1.1*w(i)]);
    Gre = Gre(:);
    Gim = Gim(:);
    h=arrowhead(Gre(1),Gim(1),Gre(2),Gim(2),0.05,[],'b');
    
    
    [Gre,Gim] = nyquist(G,[w(i) 0.9*w(i)]);
    Gre = Gre(:);
    Gim = Gim(:);
    h=arrowhead(Gre(1),-Gim(1),Gre(2),-Gim(2),0.05,[],'r');
end


%% Highlight data points at certain frequencies

% Positive Frequencies
w = [0 2 4 8 100];         % Frequencies to simulate, rad/sec
Nw = numel(w);

[Gre,Gim,w] = nyquist(G,w);
Gre = Gre(:);
Gim = Gim(:);

hold on;
plot(Gre,Gim,'bo');
hold off;

th(1) = text(Gre(1)-0.07,Gim(1),'G(0)');
th(2) = text(Gre(2)-0.06,Gim(2)+0.03,'G(2j)');
th(3) = text(Gre(3)-0.03,Gim(3)+0.04,'G(4j)');
th(4) = text(Gre(4)+0.01,Gim(4)+0.03,'G(8j)');
th(5) = text(Gre(5)+0.015,Gim(5)-0.015,'G(100j)');

% Negative Frequencies
[Gre,Gim,w] = nyquist(G,-w(2:end));
Gre = Gre(:);
Gim = Gim(:);

hold on;
plot(Gre,Gim,'ro');
hold off;

th(6) = text(Gre(1)-0.06,Gim(1)-0.03,'G(-2j)');
th(7) = text(Gre(2)-0.03,Gim(2)-0.04,'G(-4j)');
th(8) = text(Gre(3)+0.01,Gim(3)-0.03,'G(-8j)');
th(9) = text(Gre(4)+0.015,Gim(4)+0.015,'G(-100j)');

lh = legend('\omega>=0','\omega<0','Location',[0.015 0.03 1 1]); %,'Location','Best')


%% Evaluate G on a grid of frequencies 
wgrid = [0 2 4 8 100 inf]';
Ngrid = numel(wgrid);
Ggrid = zeros(Ngrid,1);
for i=1:Ngrid
    wi = wgrid(i);
    Ggrid(i) = evalfr(G,j*wi);
end

% Display magnitude (actual), phase (rad), real, imag
Gmag = abs(Ggrid);
Gph = angle(Ggrid);

[wgrid Gmag Gph real(Ggrid) imag(Ggrid)]


%% Save Plot
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure    
    
    figure(2)
    garyfyFigure        
    axis equal
    set(th,'FontSize',14);
    myprint(2,'NyquistExample')
end
