%% Nyquist Plot: First Order Systems
% This file generates a Nyquist plot for a first order system using the
% Matlab command "nyquist".


%% Nyquist Plot: First-Order System (no zero)

% System
b0 = 2;
a0 = -4;
G = tf(b0,[1 a0]);

% Use Nyquist command to compute real and imaginary data. Use this
% data to create own Nyquist plot.
% Note: The "nyquist" command returns the real and imaginary parts in a 
% three-dimensional array with dimensions Ny-by-Nu-by-Nw where Ny/Nu/Nw
% are the number of outputs, inputs, and frequencies.  For single-input
% single-output systems this is 1-by-1-by-Nw.  The ":" operation, e.g.
% Gre(:), converts this data into an Nw-by-1 column vector.
[Gre,Gim,ww] = nyquist(G);
Gre = Gre(:);
Gim = Gim(:);

figure(1)
clf;
plot(Gre,Gim,'b',Gre,-Gim,'r-.')
xlabel('Real Part');
ylabel('Imag. Part');
title('Nyquist Diagram');
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot arrowheads at certain frequencies
w = [1 16];
for i=1:numel(w)
    [Gre,Gim] = nyquist(G,[w(i) 1.1*w(i)]);
    Gre = Gre(:);
    Gim = Gim(:);
    h=arrowhead(Gre(1),Gim(1),Gre(2),Gim(2),0.05,[],'b');
    
    
    [Gre,Gim] = nyquist(G,[w(i) 0.9*w(i)]);
    Gre = Gre(:);
    Gim = Gim(:);
    h=arrowhead(Gre(1),-Gim(1),Gre(2),-Gim(2),0.05,[],'r');
end

% Highlight data points at certain frequencies
w = [0 4 inf];         % Frequencies to simulate, rad/sec
Nw = numel(w);

[Gre,Gim,w] = nyquist(G,w);
Gre = Gre(:);
Gim = Gim(:);
hold on;
plot(Gre,Gim,'bo');
hold off;

th1 = [];
th1(1) = text(Gre(1)+0.02,Gim(1),'G(0j)');
th1(2) = text(Gre(2)-0.02,Gim(2)+0.04,'G(4j)');
th1(3) = text(Gre(3)-0.1,Gim(3),'G(j\infty)');

lh = legend('\omega>=0','\omega<0','Location',[0.015 0.03 1 1]); %,'Location','Best')

%% Nyquist Plot: First-Order System With Zero

% System
b1 = 3;
b0 = 5;
a0 = -2;
G = tf([b1 b0],[1 a0]);

% Use Nyquist command to compute real and imaginary data. Use this
% data to create own Nyquist plot.
% Note: The "nyquist" command returns the real and imaginary parts in a 
% three-dimensional array with dimensions Ny-by-Nu-by-Nw where Ny/Nu/Nw
% are the number of outputs, inputs, and frequencies.  For single-input
% single-output systems this is 1-by-1-by-Nw.  The ":" operation, e.g.
% Gre(:), converts this data into an Nw-by-1 column vector.
[Gre,Gim,ww] = nyquist(G);
Gre = Gre(:);
Gim = Gim(:);

figure(2)
clf;
plot(Gre,Gim,'b',Gre,-Gim,'r-.')
xlabel('Real Part');
ylabel('Imag. Part');
title('Nyquist Diagram');
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot arrowheads at certain frequencies
w = [0.5 8];
for i=1:numel(w)
    [Gre,Gim] = nyquist(G,[w(i) 1.1*w(i)]);
    Gre = Gre(:);
    Gim = Gim(:);
    h=arrowhead(Gre(1),Gim(1),Gre(2),Gim(2),0.5,[],'b');
    
    
    [Gre,Gim] = nyquist(G,[w(i) 0.9*w(i)]);
    Gre = Gre(:);
    Gim = Gim(:);
    h=arrowhead(Gre(1),-Gim(1),Gre(2),-Gim(2),0.5,[],'r');
end

% Highlight data points at certain frequencies
w = [0 2 inf];         % Frequencies to simulate, rad/sec
Nw = numel(w);

[Gre,Gim,w] = nyquist(G,w);
Gre = Gre(:);
Gim = Gim(:);
hold on;
plot(Gre,Gim,'bo');
hold off;

th2 = [];
th2(1) = text(Gre(1)+0.2,Gim(1),'G(0j)');
th2(2) = text(Gre(2)-0.25,Gim(2)+0.4,'G(2j)');
th2(3) = text(Gre(3)-1,Gim(3),'G(j\infty)');

lh = legend('\omega>=0','\omega<0','Location',[0.015 0.03 1 1]); %,'Location','Best')

%% Save Plots
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure        
    axis equal
    set(th1,'FontSize',14);
    myprint(1,'NyquistFO1')

    figure(2)
    garyfyFigure        
    axis equal
    set(th2,'FontSize',14);
    myprint(2,'NyquistFO2')
end
